// Copyright (c) 1998, 1999, 2000 Jan Luebbe. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Jan Luebbe can be emailed at jluebbe@lasnet.de

unit Tools;

{
  3.12.1999 longint --> longword in IntToSpacedHex und SpacedHexToInt
}

interface

uses
  Graphics;

const
  NewLineString: string = #13#10;

type
  ByteArray2 = array[1..2] of Byte;
  ByteArray4 = array[1..4] of Byte;

function IntToSpacedHex(Number: longword; Count: byte): string;
function IntToByte(Number: longint; Byte: byte): byte;
function SpacedHexToInt(Number: string; Count: byte): longword;
function UOColorToRGB(UOColor: word): TColor;
function Bit(Input: byte; Pos: byte): byte;
function ByteBitField(Input: byte): string;
function WordBitField(Input: word): string; //Untested

implementation

uses
  SysUtils;

function IntToSpacedHex(Number: longword; Count: byte): string;
var
  Buffer: array[1..4] of byte; {1 hochwertig, 4 niedrigwertig}
begin
  move(Number, Buffer, 4);
  if Count > 0 then
    Result := IntToHex(Buffer[1], 2);
  if Count > 1 then
    Result := IntToHex(Buffer[2], 2) + ' ' + Result;
  if Count > 2 then
    Result := IntToHex(Buffer[3], 2) + ' ' + Result;
  if Count > 3 then
    Result := IntToHex(Buffer[4], 2) + ' ' + Result;
end;

function IntToByte(Number: longint; Byte: byte): byte;
begin
  case Byte of
    1: Result := ByteArray4(Number)[4];
    2: Result := ByteArray4(Number)[3];
    3: Result := ByteArray4(Number)[2];
    4: Result := ByteArray4(Number)[1];
  else
    Result := 0;
  end;
end;

function SpacedHexToInt(Number: string; Count: byte): longword;
var
  Buffer: array[1..4] of byte; {1 hochwertig, 4 niedrigwertig}
  i: byte;
  Ok: boolean;
begin
  Ok := True;
  Number := UpperCase(Number);
  Trim(Number);
  while Pos(' ', Number) > 0 do
    Delete(Number, Pos(' ', Number), 1);
  if Length(Number) = Count*2 then
  begin
    for i := 1 to Length(Number) do
    begin
      if Odd(i) then
        case Number[i] of
          '0': Buffer[(i+1) div 2] :=  0 * 16;
          '1': Buffer[(i+1) div 2] :=  1 * 16;
          '2': Buffer[(i+1) div 2] :=  2 * 16;
          '3': Buffer[(i+1) div 2] :=  3 * 16;
          '4': Buffer[(i+1) div 2] :=  4 * 16;
          '5': Buffer[(i+1) div 2] :=  5 * 16;
          '6': Buffer[(i+1) div 2] :=  6 * 16;
          '7': Buffer[(i+1) div 2] :=  7 * 16;
          '8': Buffer[(i+1) div 2] :=  8 * 16;
          '9': Buffer[(i+1) div 2] :=  9 * 16;
          'A': Buffer[(i+1) div 2] := 10 * 16;
          'B': Buffer[(i+1) div 2] := 11 * 16;
          'C': Buffer[(i+1) div 2] := 12 * 16;
          'D': Buffer[(i+1) div 2] := 13 * 16;
          'E': Buffer[(i+1) div 2] := 14 * 16;
          'F': Buffer[(i+1) div 2] := 15 * 16;
        else
          Ok := False;
        end
      else
        case Number[i] of
          '0': Buffer[(i+1) div 2] :=  0 + Buffer[(i+1) div 2];
          '1': Buffer[(i+1) div 2] :=  1 + Buffer[(i+1) div 2];
          '2': Buffer[(i+1) div 2] :=  2 + Buffer[(i+1) div 2];
          '3': Buffer[(i+1) div 2] :=  3 + Buffer[(i+1) div 2];
          '4': Buffer[(i+1) div 2] :=  4 + Buffer[(i+1) div 2];
          '5': Buffer[(i+1) div 2] :=  5 + Buffer[(i+1) div 2];
          '6': Buffer[(i+1) div 2] :=  6 + Buffer[(i+1) div 2];
          '7': Buffer[(i+1) div 2] :=  7 + Buffer[(i+1) div 2];
          '8': Buffer[(i+1) div 2] :=  8 + Buffer[(i+1) div 2];
          '9': Buffer[(i+1) div 2] :=  9 + Buffer[(i+1) div 2];
          'A': Buffer[(i+1) div 2] := 10 + Buffer[(i+1) div 2];
          'B': Buffer[(i+1) div 2] := 11 + Buffer[(i+1) div 2];
          'C': Buffer[(i+1) div 2] := 12 + Buffer[(i+1) div 2];
          'D': Buffer[(i+1) div 2] := 13 + Buffer[(i+1) div 2];
          'E': Buffer[(i+1) div 2] := 14 + Buffer[(i+1) div 2];
          'F': Buffer[(i+1) div 2] := 15 + Buffer[(i+1) div 2];
        else
          Ok := False;
        end;
    end;
  end
  else
    Ok := False;
  if Count = 2 then
  begin
    Buffer[3] := Buffer[1];
    Buffer[4] := Buffer[2];
    Buffer[1] := 0;
    Buffer[2] := 0;
  end;
  if Ok = True then
  begin
    ByteArray4(Result)[1] := Buffer[4];
    ByteArray4(Result)[2] := Buffer[3];
    ByteArray4(Result)[3] := Buffer[2];
    ByteArray4(Result)[4] := Buffer[1];
  end
  else
    raise EConvertError.Create('Input not SpacedHex');
end;

function UOColorToRGB(UOColor: word): TColor;
var
  B1, B2: byte;
  R, G, B: byte;
  {LongTemp: longint;
  ShortTemp: byte;}
begin {this if for converting a 15 bit color to 24 bit RGB}
  B1 := ByteArray2(UOColor)[2]; {First Byte in B1}
  B2 := ByteArray2(UOColor)[1]; {Second Byte in B2}

  {B1 := $21;
  B2 := $08;}

  R := B1 and $F8; {red}              {11111000}

  G := B1 and $07; {green}             {00000111}
  G := G shl 3;
  G := G or ((B2 and $E0) shr 5);     {11100000}
  G := G shl 3;

  B := B2 and $1F; {blue}             {00011111}
  B := B shl 3;

  {LongTemp := (R+G+B) div 3;
  ShortTemp := LongTemp;}

  ByteArray4(Result)[1] := R; {red}
  ByteArray4(Result)[2] := G; {green}
  ByteArray4(Result)[3] := B; {blue}
  ByteArray4(Result)[4] := $00; {????}
end;

function Bit(Input: byte; Pos: byte): byte;
var
  Mask: byte;
begin
  if (Pos < 1) or (Pos > 8) then
  begin
    Result  := $FF;
    Exit;
  end;
  Mask := $80;                        {10000000}
  Mask := Mask shr (Pos-1);             {Pos = 5; Mask := 00001000}
  Result := Input and Mask;           {Input = 11011110; Result := 00001000}
  Result := Result shr (8-Pos);
end;

function ByteBitField(Input: byte): string;
var
  i: byte;
begin
  Result := '';
  for i := 1 to 8 do Result := Result+IntToStr(Bit(Input, i));
end;

function WordBitField(Input: word): string; //Untested
var
  i: byte;
  Buffer: word;
begin
  Result := '';
  Buffer := Input and $FF00;
  for i := 1 to 8 do Result := Result+IntToStr(Bit(Buffer, i));
  Buffer := Input and $00FF;
  for i := 1 to 8 do Result := Result+IntToStr(Bit(Buffer, i));
end;

end.

